searchData={"items":[{"type":"module","title":"Phantom","doc":"MCP (Model Context Protocol) framework for Elixir Plug.\n\nThis library provides a complete implementation of the [MCP server specification](https://modelcontextprotocol.io/specification/2025-03-26/basic/transports) with Plug.","ref":"Phantom.html"},{"type":"module","title":"Installation - Phantom","doc":"Add Phantom to your dependencies:\n\n```elixir\n  {:phantom_mcp, \"~> 0.3.0\"},\n```\n\nWhen using with Plug/Phoenix, configure MIME to accept SSE:\n\n```elixir\n# config/config.exs\nconfig :mime, :types, %{\n  \"text/event-stream\" => [\"sse\"]\n}\n```\n\nFor Streamable HTTP access to your MCP server, forward\na path from your Plug or Phoenix Router to your MCP router.\n\n<!-- tabs-open -->","ref":"Phantom.html#module-installation"},{"type":"module","title":"Phoenix - Phantom","doc":"```elixir\ndefmodule MyAppWeb.Router do\n  use MyAppWeb, :router\n  # ...\n\n  pipeline :mcp do\n    plug :accepts, [\"json\", \"sse\"]\n\n    plug Plug.Parsers,\n      parsers: [{:json, length: 1_000_000}],\n      pass: [\"application/json\"],\n      json_decoder: JSON\n  end\n\n  scope \"/mcp\" do\n    pipe_through :mcp\n\n    forward \"/\", Phantom.Plug,\n      router: MyApp.MCPRouter\n  end\nend\n```","ref":"Phantom.html#module-phoenix"},{"type":"module","title":"Plug.Router - Phantom","doc":"```elixir\ndefmodule MyAppWeb.Router do\n  use Plug.Router\n\n  plug :match\n\n  plug Plug.Parsers,\n    parsers: [{:json, length: 1_000_000}],\n    pass: [\"application/json\"],\n    json_decoder: JSON\n\n  plug :dispatch\n\n  forward \"/mcp\",\n    to: Phantom.Plug,\n    init_opts: [\n      router: MyApp.MCP.Router\n    ]\nend\n```\n\n<!-- tabs-close -->\n\nNow the fun begins: it's time to define your MCP router that catalogs all your tools, prompts, and resources. When you're creating your MCP server, make sure\nyou test it with the [MCP Inspector](https://modelcontextprotocol.io/docs/tools/inspector).\n\nFor local testing, you can use [`mcp-remote`](https://github.com/geelen/mcp-remote) to proxy local-only clients to your Phantom-powered MCP server, either while it's hosted locally or remotely. Don't use `mcp-proxy` since it's designed for older SSE-based MCP servers (Phantom is using the newer Streamable HTTP behavior).\n\nWe'll go through each one and show how to respond synchronously or asynchronously.","ref":"Phantom.html#module-plug-router"},{"type":"module","title":"Defining Tools - Phantom","doc":"You can define tools that have an optional input_schema and an optional output_schema. If no schema is provided, then the client will not know to send arguments to your handlers.\n\n```elixir\n# Defining available tools\n# Be mindful, the input_schema is not validated upon requests.\n@description \"\"\"\nCreate a question for the provided Study.\n\"\"\"\ntool :create_question, MyApp.MCP,\n  input_schema: %{\n    required: ~w[description label study_id],\n    properties: %{\n      study_id: %{\n        type: \"integer\",\n        description: \"The unique identifier for the Study\"\n      },\n      label: %{\n        type: \"string\",\n        description: \"The title of the Question. The first thing the participant will see when presented with the question\"\n      },\n      description: %{\n        type: \"string\",\n        description: \"The contents of the question. About one paragraph of detail that defines one question or task for the participant to perform or answer\"\n      }\n    }\n  }\n```\n\nThen implement it:\n\n<!-- tabs-open -->","ref":"Phantom.html#module-defining-tools"},{"type":"module","title":"Synchronously - Phantom","doc":"```elixir\nrequire Phantom.Tool, as: Tool\n\ndef create_question(%{\"study_id\" => study_id} = params, session) do\n  changeset = MyApp.Question.changeset(%Question{}, params)\n  with {:ok, question} <- MyApp.Repo.insert(changeset),\n       {:ok, uri, resource_template} <-\n            MyApp.MCP.Router.resource_for(session, :question, id: question.id) do\n    {:reply, Tool.resource_link(uri, resource_template), session}\n  else\n    _ -> {:reply, Tool.error(\"Invalid paramaters\"), session}\n  end\nend\n```","ref":"Phantom.html#module-synchronously"},{"type":"module","title":"Asynchronously - Phantom","doc":"```elixir\nrequire Phantom.Tool, as: Tool\n\ndef create_question(%{\"study_id\" => study_id} = params, session) do\n  Task.async(fn ->\n    Process.sleep(1000)\n    changeset = MyApp.Question.changeset(%Question{}, params)\n    with {:ok, question} <- MyApp.Repo.insert(changeset),\n        {:ok, uri, resource_template} <-\n              MyApp.MCP.Router.resource_for(session, :question, id: question.id) do\n\n      Session.respond(session, Tool.resource_link(uri, resource_template))\n    else\n      _ ->  Session.respond(session, Tool.error(\"Invalid paramaters\")))\n    end\n  end)\n  {:noreply, session}\nend\n```\n\n<!-- tabs-close -->","ref":"Phantom.html#module-asynchronously"},{"type":"module","title":"Defining Prompts - Phantom","doc":"```elixir\ndefmodule MyApp.MCP.Router do\n  use Phantom.Router,\n    name: \"MyApp\",\n    vsn: \"1.0\"\n\n  # Prompts may contain arguments. If there are arguments\n  # you may want to also provide a completion function to\n  # help the client fill in the argument.\n\n  @description \"\"\"\n  Review the provided Study and provide meaningful feedback about the study and let me know if there are gaps or missing questions. We want\n  a meaningful study that can provide insight to the research goals stated\n  in the study.\n  \"\"\"\n  prompt :suggest_questions,\n    completion_function: :study_complete,\n    arguments: [\n      %{\n        name: \"study_id\",\n        description: \"The study to review\",\n        required: true\n      }\n    ]\n```\n\nThen implement it\n\n<!-- tabs-open -->","ref":"Phantom.html#module-defining-prompts"},{"type":"module","title":"Synchronously - Phantom","doc":"```elixir\nrequire Phantom.Prompt, as: Prompt\n\ndef suggest_questions(%{\"study_id\" => study_id}, session) do\n  case MyApp.MCP.Router.read_resource(session, :study, id: study_id) do\n    {:ok, uri, resource} ->\n      {:reply,\n        Prompt.response(\n          assistant: Prompt.embedded_resource(uri, resource),\n          user: Prompt.text(\"Wowzers\"),\n          assistant: Prompt.image(File.read!(\"foo.png\")),\n          user: Prompt.text(\"Seriously, wowzers\")\n        ), session}\n\n    error ->\n      {:error, Phantom.Request.internal_error(), session}\n  end\nend\n```","ref":"Phantom.html#module-synchronously-1"},{"type":"module","title":"Asynchronously - Phantom","doc":"```elixir\nrequire Phantom.Prompt, as: Prompt\n\ndef suggest_questions(%{\"study_id\" => study_id}, session) do\n  Task.async(fn ->\n    case MyApp.MCP.Router.read_resource(session, :study, id: study_id) do\n      {:ok, uri, resource} ->\n        Session.respond(session, Prompt.response(\n          assistant: Prompt.embedded_resource(uri, resource),\n          user: Prompt.text(\"Wowzers\"),\n          assistant: Prompt.image(File.read!(\"foo.png\")),\n          user: Prompt.text(\"Seriously, wowzers\")\n        ))\n\n      error ->\n        Session.respond(session, Phantom.Request.internal_error())\n    end\n  end)\n  {:noreply, sessin}\nend\n```\n\n<!-- tabs-close -->","ref":"Phantom.html#module-asynchronously-1"},{"type":"module","title":"Defining Resources - Phantom","doc":"Let's define a resource with a resource template:\n\n```elixir\n@description \"\"\"\nRead the cover image of a Study to gain some context of the\naudience, research goals, and questions.\n\"\"\"\nresource \"myapp:///studies/:study_id/cover\", :study_cover,\n  completion_function: :study_complete,\n  mime_type: \"image/png\"\n\n@description \"\"\"\nRead the contents of a study. This includes the questions and general\ncontext, which is helpful for understanding research goals.\n\"\"\"\nresource \"https://example.com/studies/:study_id/md\", :study,\n  completion_function: :study_complete,\n  mime_type: \"text/markdown\"\n```\n\nThen implement them:\n\n<!-- tabs-open -->","ref":"Phantom.html#module-defining-resources"},{"type":"module","title":"Synchronously - Phantom","doc":"```elixir\nrequire Phantom.Resource, as: Resource\n\ndef study(%{\"study_id\" => id} = params, session) do\n  study = Repo.get(Study, id)\n  text = Study.to_markdown(study)\n  {:reply, Resource.text(text), session}\nend\n\ndef study_cover(%{\"study_id\" => id} = params, session) do\n  study = Repo.get(Study, id)\n  blob = File.read!(study.cover)\n  {:reply, Resource.blob(blob), session}\nend","ref":"Phantom.html#module-synchronously-2"},{"type":"module","title":"Implement the completion handler: - Phantom","doc":"import Ecto.Query\n\ndef study_complete(\"study_id\", value, session) do\n  study_ids = Repo.all(\n    from s in Study,\n      select: s.id,\n      where: like(type(:id, :string), \"#{value}%\"),\n      where: s.account_id == ^session.user.account_id,\n      order_by: s.id,\n      limit: 101\n    )\n\n  # You may also return a map with more info:\n  # `%{values: study_ids, has_more: true, total: 1_000_000}`\n  # If you return more than 100, then Phantom will set `has_more: true`\n  # and only return the first 100.\n  {:reply, study_ids, session}\nend\n```","ref":"Phantom.html#module-implement-the-completion-handler"},{"type":"module","title":"Asynchronously - Phantom","doc":"```elixir\nrequire Phantom.Resource, as: Resource\n\ndef study(%{\"study_id\" => id} = params, session) do\n  Task.async(fn ->\n    Process.sleep(1000)\n    study = Repo.get(Study, id)\n    text = Study.to_markdown(study)\n    Session.respond(session, Resource.response(Resource.text(text)))\n  end)\n\n  {:noreply, session}\nend\n\ndef study_cover(%{\"study_id\" => id} = params, session) do\n  Task.async(fn ->\n    Process.sleep(1000)\n    study = Repo.get(Study, id)\n    blob = File.read!(study.cover)\n    Session.respond(session, Resource.response(Resource.blob(blob)))\n  end)\n\n  {:noreply, session}\nend","ref":"Phantom.html#module-asynchronously-2"},{"type":"module","title":"Implement the completion handler: - Phantom","doc":"import Ecto.Query\n\ndef study_complete(\"study_id\", value, session) do\n  study_ids = Repo.all(\n    from s in Study,\n      select: s.id,\n      where: like(type(:id, :string), \"#{value}%\"),\n      where: s.account_id == ^session.user.account_id,\n      order_by: s.id,\n      limit: 101\n    )\n\n  # You may also return a map with more info:\n  # `%{values: study_ids, has_more: true, total: 1_000_000}`\n  # If you return more than 100, then Phantom will set `has_more: true`\n  # and only return the first 100.\n  {:reply, study_ids, session}\nend\n```\n\n<!-- tabs-close -->\n\nYou'll also want to implement `list_resources/2` in your router which is\nto provide a list of all available resources in your system and return\nresource links to them.\n\n```elixir\n@salt \"cursor\"\ndef list_resources(cursor, session) do\n  # Remember to check for allowed resources according to `session.allowed_resource_templates`\n  # Below is a toy implementation for illustrative purposes.\n  cursor =\n    if cursor do\n      {:ok, cursor} = Phoenix.Token.verify(MyApp.Endpoint, @salt, cursor)\n      cursor\n    else\n      0\n    end\n\n  {_before_cursor, after_cursor} = Enum.split_while(1..1000, fn i -> i  \n      {:ok, uri, spec} = resource_for(session, :study, id: i)\n      Resource.resource_link(uri, spec, name: \"Study #{i}\")\n    end)\n\n  {:reply,\n    Resource.list(resource_links, next_cursor),\n    session}\nend\n```\n\nYou can notify the client of resource updates in case they have subscribed\nto any updates for the resource.\n\n```elixir\n# Do some work and update some underlying resource,\n# then notify any listeners:\n{:ok, uri} = MyApp.MCP.Router.resource_uri(:my_resource, id: \"foo\")\nPhantom.Tracker.notify_resource_updated(uri)\n```","ref":"Phantom.html#module-implement-the-completion-handler-1"},{"type":"module","title":"What PhantomMCP supports - Phantom","doc":"Phantom will implement these MCP requests on your behalf:\n\n- `initialize`. Phantom will detect what capabilities are available to the client based on the provided tooling defined in the Phantom router.\n- `prompts/list` list either the allowed prompts provided in the `connect/2` callback, or all prompts by default. To disable, return `allow_prompts(session, [])` in the `connect/2` callback.\n- `prompts/get` dispatch the request to your handler if allowed. Read more in `Phantom.Prompt`.\n- `resources/list` dispatch to your MCP router. By default it will be an empty list until you implement it. Read more in `Phantom.Resource`.\n- `resource/templates/list` list either the allowed resources as provided in the `connect/2` callback or all resource templates by default. To disable, return `allow_resource_templates(session, [])` in the `connect/2` callback. Read more in `Phantom.ResourceTemplate`.\n- `resources/read` dispatch the request to your handler. `Phantom.Resource`.\n- `resources/subscribe` available if the MCP router is configured with `pubsub`. To notify of updates for the resource, use `Phantom.Tracker.notify_resource_updated(uri)`.\n- `resources/unsubscribe` see above.\n- `logging/setLevel` available if the MCP router is configured with `pubsub`. Logs can be sent to client with `Session.log_{level}(session, map_content)`. [See docs](https://modelcontextprotocol.io/specification/2025-03-26/server/utilities/logging#log-levels).\n- `tools/list` list either the allowed tools as provided in the `connect/2` callback or all tools by default. To disable, return `allow_tools(session, [])` in the `connect/2` callback.\n- `tools/call` dispatch the request to your handler. Read more in `Phantom.Tool`.\n- `completion/complete` dispatch the request to your completion handler for the given prompt or resource.\n- `notification/*` no-op.\n- `ping` pong\n- `notifications/resources/list_changed` - The server informs the client the list of resources has updated. This is not done automatically; you will need to trigger this with `Phantom.Tracker.notify_resource_list/0`, but also be mindful of what resources the session may have access to.\n- `notifications/prompts/list_changed` - The server informs the client the list of prompts has updated. This is triggered when `Phantom.Cache.add_prompt/2` is called.\n- `notifications/tools/list_changed` - The server informs the client the list of tools has updated. This is triggered when `Phantom.Cache.add_tool/2` is called.\n\nPhantom **does not yet support these methods**:\n\n- `roots/list` - The server requests the client to provide a list of files available for interaction. This is like `resources/list` but for the client.\n- `sampling/createMessage` - The server requests the client to query their LLM and provide its response. This is for human-in-the-loop agentic actions and could be leveraged when the client requests a prompt from the server.\n- `elicitation/create` - The server requests input from\n  the client in order to complete a request the client has made of it.","ref":"Phantom.html#module-what-phantommcp-supports"},{"type":"module","title":"Batched Requests - Phantom","doc":"Batched requests will also be handled transparently. **please note** there is not an abstraction for efficiently providing these as a group to your handler. Since the MCP specification is deprecating batched request support in the next version, there is no plan to make this more efficient.","ref":"Phantom.html#module-batched-requests"},{"type":"module","title":"Authentication and Authorization - Phantom","doc":"Phantom does not implement authentication on its own. MCP applications needing authentication should investigate OAuth provider solutions like [Oidcc](https://hex.pm/packages/oidcc) or [Boruta](https://hex.pm/packages/boruta) or [ExOauth2Provider](https://hex.pm/packages/ex_oauth2_provider) and configure the route to serve a discovery endpoint.\n\n1. [MCP authentication and discovery](https://modelcontextprotocol.io/specification/2025-06-18/basic/authorization) is not handled by Phantom itself. You will need to implement OAuth2 and provide the discovery mechanisms as described in the specification. In the `connect/2` callback you can return `{:unauthorized, www_authenticate_info}` or `{:forbidden, \"error message\"}` to inform the client of how to move forward. An `{:ok, session}` result will imply successful auth.\n\n2. Once the authentication flow has been completed, a request to the MCP router should land with an authorization header that can be received and verified in the `connect/2` callback of your MCP router.\n\n3. You may also decide to limit the available tools, prompts, or resources depending on your authorization rules. An example is below.\n\n```elixir\ndefmodule MyApp.MCP.Router do\n  use Phantom.Router,\n    name: \"MyApp\",\n    vsn: \"1.0\"\n\n  require Logger\n\n  def connect(session, %{headers: auth_info}) do\n    # The `auth_info` will depend on the adapter, in this case it's from\n    # Plug, so it will contain query parameters and request headers.\n    with {:ok, user} <- MyApp.authenticate(conn, auth_info),\n         {:ok, my_session_state} <- MyApp.load_session(session.id) do\n      {:ok,\n        session\n        |> assign(some_state: my_session_state, user: user)\n        |> limit_for_plan(user.plan)}\n    else\n      :not_found ->\n        # See `Phantom.Plug.www_authenticate/1`\n        {:unauthorized, %{\n          method: \"Bearer\",\n          resource_metadata:  \"https://myapp.com/.well-known/oauth-protected-resource\"\n        }}\n      :not_allowed ->\n        {:forbidden, \"Please upgrade plan to use MCP server\"}\n    end\n  end\n\n  defp limit_for_plan(session, :ultra), do: session\n  defp limit_for_plan(session, :basic) do\n    # allow-list tools by stringified name. The name is either supplied as the `name` when defining it, or the stringified function name.\n    session\n    |> Phantom.Session.allowed_tools(~w[create_question])\n    |> Phantom.Session.allowed_resource_templates(~w[study])\n  end\n```","ref":"Phantom.html#module-authentication-and-authorization"},{"type":"module","title":"Optional callbacks - Phantom","doc":"There are several optional callbacks to help you hook into the lifecycle of the connections.\n\n- `disconnect/1` means the request has closed, not that the session is finished.\n- `terminate/1` means the session has finished and the client doesn't intend to resume it.\n\nFor Telemetry, please see `m:Phantom.Plug#module-telemetry` and `m:Phantom.Router#module-telemetry` for emitted telemetry hooks.","ref":"Phantom.html#module-optional-callbacks"},{"type":"module","title":"Persistent Streams - Phantom","doc":"MCP supports SSE streams to get notifications allow resource subscriptions.\nTo support this, Phantom needs to track connection pids in your cluster and uses\nPhoenix.Tracker (`phoenix_pubsub`) to do this.\n\nMCP defines a \"Streamable HTTP\" protocol, which is typical HTTP and SSE connections but with a certain behavior. MCP will typically have multiple connections to facilitate requests:\n\n1. `POST` for every command, such as `tools/call` or `resources/read`. Phantom will open an SSE stream and then immediately close the connection once work has completed.\n2. `GET` to start an SSE stream for any events such as logs and notifications. There is no work to complete with these requests, and therefore is just a \"channel\" for receiving server requests and notifications. The connection will remain open until either the client or server closes it.\n\nAll connections may provide an `mcp-session-id` header to resume a session.\n\n**Not yet supported** is the ability to resume broken connections with missed messages with the `last-event-id` header, however this is planned to be supported with a ttl-expiring distributed circular buffer.\n\nTo make Phantom distributed, start the `Phantom.Tracker` and pass in your pubsub module to the `Phantom.Plug` options:\n\n```elixir\n# Add to your application supervision tree:\n\n{Phoenix.PubSub, name: MyApp.PubSub},\n{Phantom.Tracker, [name: Phantom.Tracker, pubsub_server: MyApp.PubSub]},\n```\n\nAdjust the Phoenix router or Plug.Router options to include the PubSub server\n\n<!-- tabs-open -->","ref":"Phantom.html#module-persistent-streams"},{"type":"module","title":"Phoenix - Phantom","doc":"```elixir\nforward \"/\", Phantom.Plug,\n  router: MyApp.MCP.Router,\n  pubsub: MyApp.PubSub\n```","ref":"Phantom.html#module-phoenix-1"},{"type":"module","title":"Plug.Router - Phantom","doc":"```elixir\nforward \"/mcp\", to: Phantom.Plug, init_opts: [\n  router: MyApp.MCP.Router,\n  pubsub: MyApp.PubSub\n]\n```\n\n<!-- tabs-close -->","ref":"Phantom.html#module-plug-router-1"},{"type":"module","title":"Phantom.Cache","doc":"Storage for tooling specifications. The backend is `:persistent_term`.\n\nYou typically won't need to interact with `Phantom.Cache` directly, but\nif you're registering new tooling within the runtime, then you may want\nto use the functions herein.","ref":"Phantom.Cache.html"},{"type":"function","title":"Phantom.Cache.add_prompt/2","doc":"Add an MCP Prompt for the given router.","ref":"Phantom.Cache.html#add_prompt/2"},{"type":"macro","title":"Phantom.Cache.add_resource_template/2","doc":"Add an MCP Resource Template for the given router.\n\nThis will also purge and generate a ResourceRouter module for each scheme\nprovided.","ref":"Phantom.Cache.html#add_resource_template/2"},{"type":"function","title":"Phantom.Cache.add_tool/2","doc":"Add an MCP Tool for the given router.","ref":"Phantom.Cache.html#add_tool/2"},{"type":"function","title":"Phantom.Cache.initialized?/1","doc":"","ref":"Phantom.Cache.html#initialized?/1"},{"type":"function","title":"Phantom.Cache.list/3","doc":"List\n\nThis will also purge and generate a ResourceRouter module for each scheme\nprovided.","ref":"Phantom.Cache.html#list/3"},{"type":"function","title":"Phantom.Cache.register/1","doc":"Initialize the cache with compiled tooling for the given router.","ref":"Phantom.Cache.html#register/1"},{"type":"module","title":"Phantom.ClientLogger","doc":"Notify the client of logs.","ref":"Phantom.ClientLogger.html"},{"type":"macro","title":"Phantom.ClientLogger.alert/2","doc":"Notify the client with a log at level \"alert\" with default domain \"server\".\nNote: this requires the `session` variable to be within scope","ref":"Phantom.ClientLogger.html#alert/2"},{"type":"function","title":"Phantom.ClientLogger.alert/3","doc":"Notify the client with a log at level \"alert\"","ref":"Phantom.ClientLogger.html#alert/3"},{"type":"macro","title":"Phantom.ClientLogger.critical/2","doc":"Notify the client with a log at level \"critical\" with default domain \"server\".\nNote: this requires the `session` variable to be within scope","ref":"Phantom.ClientLogger.html#critical/2"},{"type":"function","title":"Phantom.ClientLogger.critical/3","doc":"Notify the client with a log at level \"critical\"","ref":"Phantom.ClientLogger.html#critical/3"},{"type":"macro","title":"Phantom.ClientLogger.debug/2","doc":"Notify the client with a log at level \"debug\" with default domain \"server\".\nNote: this requires the `session` variable to be within scope","ref":"Phantom.ClientLogger.html#debug/2"},{"type":"function","title":"Phantom.ClientLogger.debug/3","doc":"Notify the client with a log at level \"debug\"","ref":"Phantom.ClientLogger.html#debug/3"},{"type":"macro","title":"Phantom.ClientLogger.emergency/2","doc":"Notify the client with a log at level \"emergency\" with default domain \"server\".\nNote: this requires the `session` variable to be within scope","ref":"Phantom.ClientLogger.html#emergency/2"},{"type":"function","title":"Phantom.ClientLogger.emergency/3","doc":"Notify the client with a log at level \"emergency\"","ref":"Phantom.ClientLogger.html#emergency/3"},{"type":"macro","title":"Phantom.ClientLogger.error/2","doc":"Notify the client with a log at level \"error\" with default domain \"server\".\nNote: this requires the `session` variable to be within scope","ref":"Phantom.ClientLogger.html#error/2"},{"type":"function","title":"Phantom.ClientLogger.error/3","doc":"Notify the client with a log at level \"error\"","ref":"Phantom.ClientLogger.html#error/3"},{"type":"macro","title":"Phantom.ClientLogger.info/2","doc":"Notify the client with a log at level \"info\" with default domain \"server\".\nNote: this requires the `session` variable to be within scope","ref":"Phantom.ClientLogger.html#info/2"},{"type":"function","title":"Phantom.ClientLogger.info/3","doc":"Notify the client with a log at level \"info\"","ref":"Phantom.ClientLogger.html#info/3"},{"type":"macro","title":"Phantom.ClientLogger.log/2","doc":"Notify the client with a log at the provided level with the provided domain.\n\nThe log contents may be structured (eg, a map) or not. If not, it will be\nwrapped into one: `%{message: your_string}`.\n\nNote: this requires the `session` variable to be within scope","ref":"Phantom.ClientLogger.html#log/2"},{"type":"macro","title":"Phantom.ClientLogger.log/3","doc":"Notify the client at the provided level for domain with the payload.\n\nNote: this requires the `session` to be within scope.","ref":"Phantom.ClientLogger.html#log/3"},{"type":"function","title":"Phantom.ClientLogger.log/4","doc":"Notify the client for the provided session and domain at level with a payload","ref":"Phantom.ClientLogger.html#log/4"},{"type":"type","title":"Phantom.ClientLogger.log_level/0","doc":"","ref":"Phantom.ClientLogger.html#t:log_level/0"},{"type":"macro","title":"Phantom.ClientLogger.notice/2","doc":"Notify the client with a log at level \"notice\" with default domain \"server\".\nNote: this requires the `session` variable to be within scope","ref":"Phantom.ClientLogger.html#notice/2"},{"type":"function","title":"Phantom.ClientLogger.notice/3","doc":"Notify the client with a log at level \"notice\"","ref":"Phantom.ClientLogger.html#notice/3"},{"type":"macro","title":"Phantom.ClientLogger.warning/2","doc":"Notify the client with a log at level \"warning\" with default domain \"server\".\nNote: this requires the `session` variable to be within scope","ref":"Phantom.ClientLogger.html#warning/2"},{"type":"function","title":"Phantom.ClientLogger.warning/3","doc":"Notify the client with a log at level \"warning\"","ref":"Phantom.ClientLogger.html#warning/3"},{"type":"module","title":"Phantom.Elicit","doc":"The Model Context Protocol (MCP) provides a standardized way for\nservers to request additional information from users through the\nclient during interactions. This flow allows clients to maintain\ncontrol over user interactions and data sharing while enabling\nservers to gather necessary information dynamically. Servers\nrequest structured data from users with JSON schemas to validate\nresponses.\n\n> #### Error {: .error}\n>\n> Note: this is not yet tested\n\nhttps://modelcontextprotocol.io/specification/2025-06-18/client/elicitation\n\n```mermaid\nsequenceDiagram\n    participant User\n    participant Client\n    participant Server\n\n    Note over Server,Client: Server initiates elicitation\n    Server->>Client: elicitation/create\n\n    Note over Client,User: Human interaction\n    Client->>User: Present elicitation UI\n    User-->>Client: Provide requested information\n\n    Note over Server,Client: Complete request\n    Client-->>Server: Return user response\n\n    Note over Server: Continue processing with new information\n```","ref":"Phantom.Elicit.html"},{"type":"type","title":"Phantom.Elicit.boolean_property/0","doc":"","ref":"Phantom.Elicit.html#t:boolean_property/0"},{"type":"function","title":"Phantom.Elicit.build/1","doc":"","ref":"Phantom.Elicit.html#build/1"},{"type":"type","title":"Phantom.Elicit.enum_property/0","doc":"","ref":"Phantom.Elicit.html#t:enum_property/0"},{"type":"type","title":"Phantom.Elicit.json/0","doc":"","ref":"Phantom.Elicit.html#t:json/0"},{"type":"type","title":"Phantom.Elicit.number_property/0","doc":"","ref":"Phantom.Elicit.html#t:number_property/0"},{"type":"type","title":"Phantom.Elicit.string_property/0","doc":"","ref":"Phantom.Elicit.html#t:string_property/0"},{"type":"type","title":"Phantom.Elicit.t/0","doc":"","ref":"Phantom.Elicit.html#t:t/0"},{"type":"function","title":"Phantom.Elicit.to_json/1","doc":"","ref":"Phantom.Elicit.html#to_json/1"},{"type":"module","title":"Phantom.Plug","doc":"Main Plug implementation for MCP HTTP transport with SSE support.\n\nThis module provides a complete MCP server implementation with:\n- JSON-RPC 2.0 message handling\n- Server-Sent Events (SSE) streaming\n- CORS handling and security features\n- Session management integration\n- Origin validation\n\n<!-- tabs-open -->","ref":"Phantom.Plug.html"},{"type":"module","title":"Phoenix - Phantom.Plug","doc":"```elixir\ndefmodule MyAppWeb.Router do\n  use MyAppWeb, :router\n  # ...\n\n  pipeline :mcp do\n    plug :accepts, [\"json\", \"sse\"]\n\n    plug Plug.Parsers,\n      parsers: [{:json, length: 1_000_000}],\n      pass: [\"application/json\"],\n      json_decoder: JSON\n  end\n\n  scope \"/mcp\" do\n    pipe_through :mcp\n\n    forward \"/\", Phantom.Plug,\n      router: MyApp.MCPRouter,\n      pubsub: MyApp.PubSub\n  end\nend\n```","ref":"Phantom.Plug.html#module-phoenix"},{"type":"module","title":"Plug.Router - Phantom.Plug","doc":"```elixir\ndefmodule MyAppWeb.Router do\n  use Plug.Router\n\n  plug :match\n\n  plug Plug.Parsers,\n      parsers: [{:json, length: 1_000_000}],\n      pass: [\"application/json\"],\n      json_decoder: JSON\n\n  plug :dispatch\n\n  forward \"/mcp\",\n      to: Phantom.Plug,\n      init_opts: [\n        router: MyApp.MCP.Router,\n        pubsub: MyApp.PubSub\n      ]\nend\n```\n\n<!-- tabs-close -->\n\nHere are the defaults:\n\n```elixir\n[\n  pubsub: nil,\n  origins: [\"http://localhost:4000\"],\n  validate_origin: true,\n  session_timeout: 30000,\n  max_request_size: 1048576\n]\n```","ref":"Phantom.Plug.html#module-plug-router"},{"type":"module","title":"Telemetry - Phantom.Plug","doc":"Telemetry is provided with these events:\n\n- `[:phantom, :plug, :request, :connect]` with meta: `~w[session router conn opts]a`\n- `[:phantom, :plug, :request, :disconnect]` with meta: `~w[session router conn]a`\n- `[:phantom, :plug, :request, :exception]` with meta: `~w[session router conn stacktrace request exception]a`","ref":"Phantom.Plug.html#module-telemetry"},{"type":"function","title":"Phantom.Plug.call/2","doc":"Callback implementation for  Plug.call/2 .","ref":"Phantom.Plug.html#call/2"},{"type":"function","title":"Phantom.Plug.init/1","doc":"Initializes the plug with the given options.","ref":"Phantom.Plug.html#init/1"},{"type":"function","title":"Options - Phantom.Plug.init/1","doc":"- `:router` - The MCP router module (required)\n- `:origins` - List of allowed origins or `:all` (default: localhost)\n- `:validate_origin` - Whether to validate Origin header (default: true)\n- `:session_timeout` - Session timeout in milliseconds (default: 30s)\n- `:max_request_size` - Maximum request size in bytes (default: 1MB)","ref":"Phantom.Plug.html#init/1-options"},{"type":"type","title":"Phantom.Plug.opts/0","doc":"","ref":"Phantom.Plug.html#t:opts/0"},{"type":"type","title":"Phantom.Plug.www_authenticate/0","doc":"","ref":"Phantom.Plug.html#t:www_authenticate/0"},{"type":"function","title":"Phantom.Plug.www_authenticate/1","doc":"Construct a WWW-Authenticate header as defined by RFC 9728 from the map.\n\nThis requires the map to contain a `:method` to indicate acceptable authentication\nmethods, typically `\"Bearer\"`, and then rest of the attributes will be serialized\ninto the header as key=value.\n\nFor example,\n\n    iex> Phantom.Plug.www_authenticate(%{\n    ...>   method: \"Bearer\",\n    ...>   resource_metadata: \"https://myapp.com/.well-known/oauth-protected-resource\",\n    ...>   max_age: 42000\n    ...> })\n    ~s|Bearer max_age=\"42000\", resource_metadata=\"https://myapp.com/.well-known/oauth-protected-resource\"|\n\nhttps://datatracker.ietf.org/doc/html/rfc9728#name-use-of-www-authenticate-for","ref":"Phantom.Plug.html#www_authenticate/1"},{"type":"module","title":"Phantom.Prompt","doc":"The Model Context Protocol (MCP) provides a standardized way\nfor servers to expose prompt templates to clients. Prompts\nallow servers to provide structured messages and instructions\nfor interacting with language models. Clients can discover\navailable prompts, retrieve their contents, and provide arguments\nto customize them.\n\n```mermaid\nsequenceDiagram\n    participant Client\n    participant Server\n\n    Note over Client,Server: Discovery\n    Client->>Server: prompts/list\n    Server-->>Client: List of prompts\n\n    Note over Client,Server: Usage\n    Client->>Server: prompts/get\n    Server-->>Client: Prompt content\n\n    opt listChanged\n      Note over Client,Server: Changes\n      Server--)Client: prompts/list_changed\n      Client->>Server: prompts/list\n      Server-->>Client: Updated prompts\n    end\n```\n\nhttps://modelcontextprotocol.io/specification/2025-03-26/server/prompts","ref":"Phantom.Prompt.html"},{"type":"function","title":"Phantom.Prompt.audio/2","doc":"Build an audio message for the prompt\n\nThe provided binary will be base64-encoded.","ref":"Phantom.Prompt.html#audio/2"},{"type":"type","title":"Phantom.Prompt.audio_content/0","doc":"","ref":"Phantom.Prompt.html#t:audio_content/0"},{"type":"function","title":"Phantom.Prompt.build/1","doc":"Build a prompt spec\n\nThe `Phantom.Router.prompt/3` macro will build these specs.\n\nFields:\n  - `:name` - The name of the prompt.\n  - `:description` - The description of the resource and when to use it.\n  - `:handler` - The module to call.\n  - `:function` - The function to call on the handler module.\n  - `:completion_function` - The function to call on the handler module that will provide possible completion results.\n  - `:arguments` - A list of arguments that the prompt takes.\n\nArgument fields:\n  - `:name` - the name of the argument, eg: \"username\"\n  - `:description` - description of the argument, eg, \"Your Github username\"\n  - `:required` - whether the argument is required in order to be called, ie: `true` or `false`","ref":"Phantom.Prompt.html#build/1"},{"type":"function","title":"Phantom.Prompt.embedded_resource/2","doc":"Embedded resource reponse.","ref":"Phantom.Prompt.html#embedded_resource/2"},{"type":"type","title":"Phantom.Prompt.embedded_resource_content/0","doc":"","ref":"Phantom.Prompt.html#t:embedded_resource_content/0"},{"type":"function","title":"Phantom.Prompt.image/2","doc":"Build an image message for the prompt\n\nThe provided binary will be base64-encoded.","ref":"Phantom.Prompt.html#image/2"},{"type":"type","title":"Phantom.Prompt.image_content/0","doc":"","ref":"Phantom.Prompt.html#t:image_content/0"},{"type":"type","title":"Phantom.Prompt.json/0","doc":"","ref":"Phantom.Prompt.html#t:json/0"},{"type":"type","title":"Phantom.Prompt.message/0","doc":"","ref":"Phantom.Prompt.html#t:message/0"},{"type":"type","title":"Phantom.Prompt.response/0","doc":"","ref":"Phantom.Prompt.html#t:response/0"},{"type":"macro","title":"Phantom.Prompt.response/1","doc":"Formats the response from an MCP Router to the MCP specification\n\nProvide a keyword list of messages with a keyword list. The key\nshould contain the role, and the value contain the message.\n\nFor example:\n\n    require Phantom.Prompt, as: Prompt\n    {:ok, uri, resource} = MyApp.MCP.Router.read_resource(session, :my_resource, 123)\n\n    Prompt.response([\n      assistant: Prompt.audio(File.read!(\"foo.wav\"), \"audio/wav\"),\n      user: Prompt.text(\"Wow that was interesting\"),\n      assistant: Prompt.image(File.read!(\"bar.png\"), \"image/png\"),\n      user: Prompt.text(\"amazing\"),\n      assistant: Prompt.embedded_resource(uri, resource)\n    ])","ref":"Phantom.Prompt.html#response/1"},{"type":"function","title":"Phantom.Prompt.response/2","doc":"Construct a prompt response with the provided messages for the given prompt\n\nSee `response/1` macro version that do the same thing but will fetch the\nprompt spec from the current session.","ref":"Phantom.Prompt.html#response/2"},{"type":"type","title":"Phantom.Prompt.t/0","doc":"","ref":"Phantom.Prompt.html#t:t/0"},{"type":"function","title":"Phantom.Prompt.text/1","doc":"Build a text message for the prompt","ref":"Phantom.Prompt.html#text/1"},{"type":"type","title":"Phantom.Prompt.text_content/0","doc":"","ref":"Phantom.Prompt.html#t:text_content/0"},{"type":"function","title":"Phantom.Prompt.to_json/1","doc":"Represent a Prompt spec as json when listing the available prompts to clients.","ref":"Phantom.Prompt.html#to_json/1"},{"type":"module","title":"Phantom.Prompt.Argument","doc":"","ref":"Phantom.Prompt.Argument.html"},{"type":"function","title":"Phantom.Prompt.Argument.build/1","doc":"Build a prompt argument spec\n\nWhen building a prompt with `Phantom.Prompt.build/1`, arguments will\nbe built automatically.","ref":"Phantom.Prompt.Argument.html#build/1"},{"type":"type","title":"Phantom.Prompt.Argument.json/0","doc":"","ref":"Phantom.Prompt.Argument.html#t:json/0"},{"type":"type","title":"Phantom.Prompt.Argument.t/0","doc":"","ref":"Phantom.Prompt.Argument.html#t:t/0"},{"type":"function","title":"Phantom.Prompt.Argument.to_json/1","doc":"Represent a Prompt argument spec as json when listing the available prompts to clients.","ref":"Phantom.Prompt.Argument.html#to_json/1"},{"type":"module","title":"Phantom.Resource","doc":"The Model Context Protocol (MCP) provides a standardized way for\nservers to expose resources to clients. Resources allow servers to\nshare data that provides context to language models, such as files,\ndatabase schemas, or application-specific information. Each resource\nis uniquely identified by a URI.\n\nhttps://modelcontextprotocol.io/specification/2025-03-26/server/resources\n\n```mermaid\nsequenceDiagram\n    participant Client\n    participant Server\n\n    Note over Client,Server: Resource Discovery\n    Client->>Server: resources/list\n    Server-->>Client: List of resources\n\n    Note over Client,Server: Resource Access\n    Client->>Server: resources/read\n    Server-->>Client: Resource contents\n\n    Note over Client,Server: Subscriptions\n    Client->>Server: resources/subscribe\n    Server-->>Client: Subscription confirmed\n\n    Note over Client,Server: Updates\n    Server--)Client: notifications/resources/updated\n    Client->>Server: resources/read\n    Server-->>Client: Updated contents\n```","ref":"Phantom.Resource.html"},{"type":"macro","title":"Phantom.Resource.blob/2","doc":"Resource as binary content\n\nThe `:uri` and `:mime_type` will be fetched from the current resource template\nwithin the scope of the request if not provided, but you will need to provide\nthe rest.\n\n- `binary` - Binary data. This will be base64-encoded by Phantom.\n- `:uri` (required) Unique identifier for the resource\n- `:name` (optional) The name of the resource.\n- `:title` (optional) human-readable name of the resource for display purposes.\n- `:description` (optional) Description\n- `:mime_type` (optional) MIME type\n- `:size` (optional) Size in bytes\n\nFor example:\n\n    Phantom.Resource.blob(\n      File.read!(\"foo.png\"),\n      uri: \"test://my-foos/123\",\n      mime_type: \"image/png\"\n    )","ref":"Phantom.Resource.html#blob/2"},{"type":"type","title":"Phantom.Resource.blob_content/0","doc":"","ref":"Phantom.Resource.html#t:blob_content/0"},{"type":"function","title":"Phantom.Resource.list/2","doc":"Formats the response from the MCP Router to the MCP specification for listing resources","ref":"Phantom.Resource.html#list/2"},{"type":"type","title":"Phantom.Resource.list_response/0","doc":"","ref":"Phantom.Resource.html#t:list_response/0"},{"type":"type","title":"Phantom.Resource.resource_link/0","doc":"","ref":"Phantom.Resource.html#t:resource_link/0"},{"type":"function","title":"Phantom.Resource.resource_link/3","doc":"Formats a resource_template and the provided attributes as a resource link. This is\ntypicaly used when listing resources or when tools embed a resource_link within its result.","ref":"Phantom.Resource.html#resource_link/3"},{"type":"type","title":"Phantom.Resource.response/0","doc":"","ref":"Phantom.Resource.html#t:response/0"},{"type":"function","title":"Phantom.Resource.response/1","doc":"Formats the response from an MCP Router to the MCP specification","ref":"Phantom.Resource.html#response/1"},{"type":"macro","title":"Phantom.Resource.text/2","doc":"Resource as text content\n\nThe `:uri` and `:mime_type` will be fetched from the current resource template\nwithin the scope of the request if not provided, but you will need to provide\nthe rest.\n\n- `text` - Text data. If a map, then it will be encoded into JSON and `:mime_type` will be set accordingly unless provided.\n- `:uri` (required) Unique identifier for the resource\n- `:name` (optional) The name of the resource.\n- `:title` (optional) human-readable name of the resource for display purposes.\n- `:description` (optional) Description\n- `:mime_type` (optional) MIME type. Defaults to `\"text/plain\"`\n- `:size` (optional) Size in bytes\n\nFor example:\n\n    Phantom.ResourceTemplate.text(\n      \"","ref":"Phantom.Resource.html#text/2"},{"type":"macro","title":"Why hello there\", - Phantom.Resource.text/2","doc":"uri: \"test://obi-wan-quotes/hello-there\",\n      mime_type: \"text/markdown\"\n    )\n\n    Phantom.ResourceTemplate.text(\n      %{why: \"hello there\"},\n      uri: \"test://my-foos/json\",\n      # mime_type: \"application/json\"  # set by Phantom\n    )","ref":"Phantom.Resource.html#text/2-why-hello-there"},{"type":"type","title":"Phantom.Resource.text_content/0","doc":"","ref":"Phantom.Resource.html#t:text_content/0"},{"type":"module","title":"Phantom.ResourceTemplate","doc":"The Model Context Protocol (MCP) provides a standardized way for\nservers to expose resources to clients. Resources allow servers to\nshare data that provides context to language models, such as files,\ndatabase schemas, or application-specific information. Each resource\nis uniquely identified by a URI.\n\nhttps://modelcontextprotocol.io/specification/2025-03-26/server/resources","ref":"Phantom.ResourceTemplate.html"},{"type":"function","title":"Phantom.ResourceTemplate.build/1","doc":"Build a resource_template spec\n\nThe `Phantom.Router.resource/3` macro will build these specs.\n\n- `:name` - The name of the resource template.\n- `:uri` - The URI template of the resource in the style of `Plug.Router`, including the scheme.\n  For example, you can define a path like `\"myapp:///some/path/:project_id/:id` which\n  will be parsed to include path params `%{\"project_id\" => _, \"id\" => _}`. The scheme can be\n  `\"https\"`, `\"git\"`, `\"file\"`, or custom, eg `\"myapp\"`.\n- `:description` - The description of the resource and when to use it.\n- `:handler` - The module to call.\n- `:function` - The function to call on the handler module.\n- `:completion_function` - The function to call on the handler module that will provide possible completion results.\n- `:mime_type` - the MIME type of the results.\n- `:router` - The Router module that will capture the URIs and route resources by URI to functions.\n  This is constructed by the `Phantom.Router.resource/3` macro automatically as\n  `MyApp.MyMCPRouter.ResourceRouter.{Scheme}`. The module does not need to exist at the time of\n  building it-- it will be generated when added by `Phantom.Cache.add_resource_template/2`\n  or by the `Phantom.Router.resource/3` macro.","ref":"Phantom.ResourceTemplate.html#build/1"},{"type":"type","title":"Phantom.ResourceTemplate.json/0","doc":"","ref":"Phantom.ResourceTemplate.html#t:json/0"},{"type":"type","title":"Phantom.ResourceTemplate.t/0","doc":"","ref":"Phantom.ResourceTemplate.html#t:t/0"},{"type":"function","title":"Phantom.ResourceTemplate.to_json/1","doc":"Represent a ResourceTemplate spec as json when listing the available resources to clients.","ref":"Phantom.ResourceTemplate.html#to_json/1"},{"type":"behaviour","title":"Phantom.Router","doc":"A DSL for defining MCP servers.\nThis module provides functions that define tools, resources, and prompts.\n\nSee `Phantom` for usage examples.","ref":"Phantom.Router.html"},{"type":"behaviour","title":"Telemetry - Phantom.Router","doc":"Telemetry is provided with these events:\n\n- `[:phantom, :dispatch, :start]` with meta: `~w[method params request session]a`\n- `[:phantom, :dispatch, :stop]` with meta: `~w[method params request result session]a`\n- `[:phantom, :dispatch, :exception]` with meta: `~w[method kind reason stacktrace params request session]a`","ref":"Phantom.Router.html#module-telemetry"},{"type":"callback","title":"Phantom.Router.connect/2","doc":"","ref":"Phantom.Router.html#c:connect/2"},{"type":"callback","title":"Phantom.Router.disconnect/1","doc":"","ref":"Phantom.Router.html#c:disconnect/1"},{"type":"callback","title":"Phantom.Router.dispatch_method/4","doc":"","ref":"Phantom.Router.html#c:dispatch_method/4"},{"type":"callback","title":"Phantom.Router.instructions/1","doc":"","ref":"Phantom.Router.html#c:instructions/1"},{"type":"callback","title":"Phantom.Router.list_resources/2","doc":"List resources available to the client.\n\nThis will expect the response to use `Phantom.Resource.list/2` as the result.\nYou may also want to leverage `resource_for/3` and `Phantom.Resource.resource_link/3`\nto construct the response. See `m:Phantom#module-defining-resources` for an exmaple.\n\nRemember to check for allowed resources according to `session.allowed_resource_templates`","ref":"Phantom.Router.html#c:list_resources/2"},{"type":"macro","title":"Phantom.Router.prompt/2","doc":"See prompt/3","ref":"Phantom.Router.html#prompt/2"},{"type":"macro","title":"Phantom.Router.prompt/3","doc":"Define a prompt that can be retrieved by the MCP client.","ref":"Phantom.Router.html#prompt/3"},{"type":"macro","title":"Examples - Phantom.Router.prompt/3","doc":"prompt :summarize,\n      description: \"A text prompt\",\n      completion_function: :summarize_complete,\n      arguments: [\n        %{\n          name: \"text\",\n          description: \"The text to summarize\",\n        },\n        %{\n          name: \"resource\",\n          description: \"The resource to summarize\",\n        }\n      ]\n    )\n\n    # ...\n\n    require Phantom.Prompt, as: Prompt\n    def summarize(args, _request, session) do\n      {:reply, Prompt.response([\n        assistant: Prompt.text(\"You're great\"),\n        user: Prompt.text(\"No you're great!\")\n      ], session}\n    end\n\n    def summarize_complete(\"text\", _typed_value, session) do\n      {:reply, [\"many values\"], session}\n    end\n\n    def summarize_complete(\"resource\", _typed_value, session) do\n      # list of IDs\n      {:reply, [\"123\"], session}\n    end","ref":"Phantom.Router.html#prompt/3-examples"},{"type":"function","title":"Phantom.Router.read_resource/3","doc":"Reads the resource given its URI, primarily for embedded resources.\n\nThis is available on your router as: `MyApp.MCP.Router.read_resource(session, name, params)`\n\nFor example:\n\n```elixir\niex> MyApp.MCP.Router.read_resource(session, :my_resource, id: 321)\n#=> {:ok, \"myapp:///resources/123\", %{\n#   blob: \"abc123\"\n#   uri: \"myapp:///resources/123\",\n#   mimeType: \"audio/wav\",\n#   name: \"Some audio\",\n#   title: \"Super audio\"\n# }\n```","ref":"Phantom.Router.html#read_resource/3"},{"type":"macro","title":"Phantom.Router.resource/2","doc":"See `resource/4`","ref":"Phantom.Router.html#resource/2"},{"type":"macro","title":"Phantom.Router.resource/4","doc":"Define a resource that can be read by the MCP client.","ref":"Phantom.Router.html#resource/4"},{"type":"macro","title":"Examples - Phantom.Router.resource/4","doc":"resource \"app:///studies/:id\", MyApp.MCP, :read_study,\n      description: \"A study\",\n      mime_type: \"application/json\"\n\n    # ...\n\n    require Phantom.Resource, as: Resource\n    def read_study(%{\"id\" => id}, _request, session) do\n      {:reply, Response.response(\n        Response.text(\"IO.puts \"Hi\"\")\n      ), session}\n    end","ref":"Phantom.Router.html#resource/4-examples"},{"type":"function","title":"Phantom.Router.resource_uri/3","doc":"Constructs a response map for the given resource with the provided parameters. This\nfunction is provided to your MCP Router that accepts the session instead.\n\nFor example\n\n```elixir\niex> MyApp.MCP.Router.resource_uri(session, :my_resource, id: 123)\n{:ok, \"myapp:///my-resource/123\"}\n\niex> MyApp.MCP.Router.resource_uri(session, :my_resource, foo: \"error\")\n{:error, \"Parameters don't match resource.\"}\n\niex> MyApp.MCP.Router.resource_uri(session, :unknown, id: 123)\n{:error, \"Router not found for resource\"}\n```","ref":"Phantom.Router.html#resource_uri/3"},{"type":"callback","title":"Phantom.Router.server_info/1","doc":"","ref":"Phantom.Router.html#c:server_info/1"},{"type":"callback","title":"Phantom.Router.terminate/1","doc":"","ref":"Phantom.Router.html#c:terminate/1"},{"type":"macro","title":"Phantom.Router.tool/2","doc":"See `tool/3`","ref":"Phantom.Router.html#tool/2"},{"type":"macro","title":"Phantom.Router.tool/3","doc":"Define a tool that can be called by the MCP client.","ref":"Phantom.Router.html#tool/3"},{"type":"macro","title":"Examples - Phantom.Router.tool/3","doc":"tool :local_echo,\n      description: \"A test that echos your message\",\n      # or supply a `@description` before defining the tool\n      input_schema: %{\n        required: [:message],\n        properties: %{\n          message: %{\n            type: \"string\",\n            description: \"message to echo\"\n          }\n        }\n      }","ref":"Phantom.Router.html#tool/3-examples"},{"type":"macro","title":"handled by your function syncronously: - Phantom.Router.tool/3","doc":"require Phantom.Tool, as: Tool\n    def local_echo(params, session) do\n      # Maps will be JSON-encoded and also provided\n      # as structured content.\n      {:reply, Tool.text(params), session}\n    end\n\n    # Or asyncronously:\n\n    require Phantom.Tool, as: Tool\n    def local_echo(params, session) do\n      Task.async(fn ->\n        Process.sleep(1000)\n        Session.respond(session, Tool.text(params))\n      end)\n\n      {:noreply, session}\n    end","ref":"Phantom.Router.html#tool/3-handled-by-your-function-syncronously"},{"type":"module","title":"Phantom.Session","doc":"Represents the state of the MCP session. This is the state across the conversation\nand is the bridge between the various transports (HTTP, stdio) to persistence,\neven if stateless.","ref":"Phantom.Session.html"},{"type":"function","title":"Phantom.Session.allow_prompts/2","doc":"Set an allow-list of usable Prompts for the session","ref":"Phantom.Session.html#allow_prompts/2"},{"type":"function","title":"Phantom.Session.allow_resource_templates/2","doc":"Set an allow-list of usable Resource Templates for the session","ref":"Phantom.Session.html#allow_resource_templates/2"},{"type":"function","title":"Phantom.Session.allow_tools/2","doc":"Set an allow-list of usable Tools for the session","ref":"Phantom.Session.html#allow_tools/2"},{"type":"function","title":"Phantom.Session.assign/2","doc":"Assign state to the session.","ref":"Phantom.Session.html#assign/2"},{"type":"function","title":"Phantom.Session.assign/3","doc":"Assign state to the session.","ref":"Phantom.Session.html#assign/3"},{"type":"function","title":"Phantom.Session.elicit/2","doc":"Elicit input from the client","ref":"Phantom.Session.html#elicit/2"},{"type":"function","title":"Phantom.Session.finish/1","doc":"Closes the connection for the session","ref":"Phantom.Session.html#finish/1"},{"type":"function","title":"Phantom.Session.list_resource_subscriptions/1","doc":"","ref":"Phantom.Session.html#list_resource_subscriptions/1"},{"type":"function","title":"Phantom.Session.list_streams/0","doc":"List all sessions with streams open.","ref":"Phantom.Session.html#list_streams/0"},{"type":"function","title":"Phantom.Session.new/2","doc":"Builds a new session with the provided session ID.\n\nThis is used for adapters such as `Phantom.Plug`. If a\nsession ID is not provided, it will generate one using `UUIDv7`.","ref":"Phantom.Session.html#new/2"},{"type":"function","title":"Phantom.Session.notify/2","doc":"Send a notification to the client","ref":"Phantom.Session.html#notify/2"},{"type":"function","title":"Phantom.Session.notify_progress/3","doc":"Send a progress notification to the client","ref":"Phantom.Session.html#notify_progress/3"},{"type":"function","title":"Phantom.Session.notify_progress/4","doc":"","ref":"Phantom.Session.html#notify_progress/4"},{"type":"function","title":"Phantom.Session.ping/1","doc":"Send a ping to the client","ref":"Phantom.Session.html#ping/1"},{"type":"function","title":"Phantom.Session.progress_token/1","doc":"Fetch the current progress token if provided by the client","ref":"Phantom.Session.html#progress_token/1"},{"type":"function","title":"Phantom.Session.respond/2","doc":"Sends response back to the stream\n\nThis should likely be used in conjunction with:\n\n- `Phantom.Tool.response(payload)`\n- `Phantom.Resource.response(payload)`\n- `Phantom.Prompt.response(payload)`\n\nFor example:\n\n```elixir\nsession_pid = session.pid\nrequest_id = request.id\n\nTask.async(fn ->\n  Session.respond(\n    session_pid,\n    request_id,\n    Phantom.Tool.audio(\n      File.read!(\"priv/static/game-over.wav\"),\n      mime_type: \"audio/wav\"\n    )\n  )\nend)\n```","ref":"Phantom.Session.html#respond/2"},{"type":"function","title":"Phantom.Session.respond/3","doc":"See `respond/2`","ref":"Phantom.Session.html#respond/3"},{"type":"function","title":"Phantom.Session.set_log_level/3","doc":"Sets the log level for the SSE stream.\nSets both for the current request for async tasks and the SSE stream","ref":"Phantom.Session.html#set_log_level/3"},{"type":"function","title":"Phantom.Session.subscribe_to_resource/2","doc":"Subscribe the session to a resource.\n\nThis is used by the MCP Router when the client requests to subscribe to the provided resource.","ref":"Phantom.Session.html#subscribe_to_resource/2"},{"type":"type","title":"Phantom.Session.t/0","doc":"","ref":"Phantom.Session.html#t:t/0"},{"type":"function","title":"Phantom.Session.unsubscribe_to_resource/2","doc":"Unsubscribe the session to a resource.\n\nThis is used by the MCP Router when the client requests to subscribe to the provided resource.","ref":"Phantom.Session.html#unsubscribe_to_resource/2"},{"type":"module","title":"Phantom.Tool","doc":"The Model Context Protocol (MCP) allows servers to expose tools\nthat can be invoked by language models. Tools enable models to\ninteract with external systems, such as querying databases,\ncalling APIs, or performing computations. Each tool is uniquely\nidentified by a name and includes metadata describing its schema.\n\nhttps://modelcontextprotocol.io/specification/2025-03-26/server/tools\n\n```mermaid\nsequenceDiagram\n    participant LLM\n    participant Client\n    participant Server\n\n    Note over Client,Server: Discovery\n    Client->>Server: tools/list\n    Server-->>Client: List of tools\n\n    Note over Client,LLM: Tool Selection\n    LLM->>Client: Select tool to use\n\n    Note over Client,Server: Invocation\n    Client->>Server: tools/call\n    Server-->>Client: Tool result\n    Client->>LLM: Process result\n\n    Note over Client,Server: Updates\n    Server--)Client: tools/list_changed\n    Client->>Server: tools/list\n    Server-->>Client: Updated tools\n```","ref":"Phantom.Tool.html"},{"type":"macro","title":"Phantom.Tool.audio/2","doc":"Tool response as audio content\n\nThe `:mime_type` will be fetched from the current tool within the scope of\nthe request if not provided, but you will need to provide the rest.\n\n- `binary` - Binary data.\n- `:mime_type` (optional) MIME type. Defaults to `\"application/octet-stream\"`\n\nFor example:\n\n    Phantom.Tool.audio(File.read!(\"game-over.wav\"))\n\n    Phantom.Tool.audio(\n      File.read!(\"game-over.wav\"),\n      mime_type: \"audio/wav\"\n    )","ref":"Phantom.Tool.html#audio/2"},{"type":"type","title":"Phantom.Tool.audio_response/0","doc":"","ref":"Phantom.Tool.html#t:audio_response/0"},{"type":"function","title":"Phantom.Tool.build/1","doc":"Build a tool spec. Be intentional with the name and description when defining\nthe tool since it will inform the LLM when to use the tool.\n\nThe `Phantom.Router.tool/3` macro will build these specs.\n\nFields:\n\n- `:name` - The name of the tool.\n- `:title` A human-readable title for the tool, useful for UI display.\n- `:description` - The description of the tool and when to use it.\n- `:mime_type` - the MIME type of the results.\n- `:handler` - The module to call.\n- `:function` - The function to call on the handler module.\n- `:output_schema` - the JSON schema of the results.\n- `:input_schema` - The JSON schema of the input arguments.\n- `:read_only` If `true`, indicates the tool does not modify its environment.\n- `:destructive` If `true`, the tool may perform destructive updates (only meaningful when `:read_only` is `false`).\n- `:idempotent` If `true`, calling the tool repeatedly with the same arguments has no additional effect (only meaningful when `:read_only` is `false`).\n- `:open_world` If `true`, the tool may interact with an \"open world\" of external entities.","ref":"Phantom.Tool.html#build/1"},{"type":"type","title":"Phantom.Tool.embedded_blob_resource/0","doc":"","ref":"Phantom.Tool.html#t:embedded_blob_resource/0"},{"type":"function","title":"Phantom.Tool.embedded_resource/2","doc":"Embedded resource response.\n\nTypically used with your router's `read_resource/3` function.\nSee `Phantom.Router.read_resource/3` for more information","ref":"Phantom.Tool.html#embedded_resource/2"},{"type":"type","title":"Phantom.Tool.embedded_resource_response/0","doc":"","ref":"Phantom.Tool.html#t:embedded_resource_response/0"},{"type":"type","title":"Phantom.Tool.embedded_text_resource/0","doc":"","ref":"Phantom.Tool.html#t:embedded_text_resource/0"},{"type":"function","title":"Phantom.Tool.error/1","doc":"","ref":"Phantom.Tool.html#error/1"},{"type":"type","title":"Phantom.Tool.error_response/0","doc":"","ref":"Phantom.Tool.html#t:error_response/0"},{"type":"macro","title":"Phantom.Tool.image/2","doc":"Tool response as image content\n\nThe `:mime_type` will be fetched from the current tool within the scope of\nthe request if not provided, but you will need to provide the rest.\n\n- `binary` - Binary data.\n- `:mime_type` (optional) MIME type. Defaults to `\"application/octet-stream\"`\n\nFor example:\n\n    Phantom.Tool.image(File.read!(\"tower.png\"))\n\n    Phantom.Tool.audio(\n      File.read!(\"tower.png\"),\n      mime_type: \"image/png\"\n    )","ref":"Phantom.Tool.html#image/2"},{"type":"type","title":"Phantom.Tool.image_response/0","doc":"","ref":"Phantom.Tool.html#t:image_response/0"},{"type":"type","title":"Phantom.Tool.json/0","doc":"","ref":"Phantom.Tool.html#t:json/0"},{"type":"function","title":"Phantom.Tool.resource_link/3","doc":"Resource link reponse.\n\nTypically used with your router's `resource_uri/3` function.\nSee `Phantom.Router.resource_uri/3` for more information.","ref":"Phantom.Tool.html#resource_link/3"},{"type":"type","title":"Phantom.Tool.resource_link_response/0","doc":"","ref":"Phantom.Tool.html#t:resource_link_response/0"},{"type":"type","title":"Phantom.Tool.response/0","doc":"","ref":"Phantom.Tool.html#t:response/0"},{"type":"function","title":"Phantom.Tool.response/1","doc":"Formats the response from an MCP Router to the MCP specification","ref":"Phantom.Tool.html#response/1"},{"type":"type","title":"Phantom.Tool.structured_response/0","doc":"","ref":"Phantom.Tool.html#t:structured_response/0"},{"type":"type","title":"Phantom.Tool.t/0","doc":"","ref":"Phantom.Tool.html#t:t/0"},{"type":"function","title":"Phantom.Tool.text/1","doc":"","ref":"Phantom.Tool.html#text/1"},{"type":"type","title":"Phantom.Tool.text_response/0","doc":"","ref":"Phantom.Tool.html#t:text_response/0"},{"type":"function","title":"Phantom.Tool.to_json/1","doc":"Represent a Tool spec as json when listing the available tools to clients.","ref":"Phantom.Tool.html#to_json/1"},{"type":"module","title":"Phantom.Tool.Annotation","doc":"Tool annotations provide additional metadata about a\ntool’s behavior, helping clients understand how to present\nand manage tools. These annotations are hints that describe\nthe nature and impact of a tool, but should not be relied\nupon for security decisions\n\n- `:title` A human-readable title for the tool, useful for UI display\n- `:read_only_hint` If true, indicates the tool does not modify its environment\n- `:destructive_hint` If true, the tool may perform destructive updates (only meaningful when `:read_only_hint` is false)\n- `:idempotent_hint` If true, calling the tool repeatedly with the same arguments has no additional effect (only meaningful when readOnlyHint is false)\n- `:open_world_hint` If true, the tool may interact with an “open world” of external entities\n\nhttps://modelcontextprotocol.io/docs/concepts/tools#tool-annotations","ref":"Phantom.Tool.Annotation.html"},{"type":"function","title":"Phantom.Tool.Annotation.build/1","doc":"","ref":"Phantom.Tool.Annotation.html#build/1"},{"type":"type","title":"Phantom.Tool.Annotation.json/0","doc":"","ref":"Phantom.Tool.Annotation.html#t:json/0"},{"type":"type","title":"Phantom.Tool.Annotation.t/0","doc":"","ref":"Phantom.Tool.Annotation.html#t:t/0"},{"type":"function","title":"Phantom.Tool.Annotation.to_json/1","doc":"","ref":"Phantom.Tool.Annotation.html#to_json/1"},{"type":"module","title":"Phantom.Tool.JSONSchema","doc":"JSON Schema representing the arguments for the tool, either as `input_schema`\nor `output_schema`.\n\nLearn more at https://json-schema.org/learn/getting-started-step-by-step\n\nExample:\n\n    %{\n      type: \"object\",\n      properties: %{\n        productId: %{\n          description: \"The unique identifier for a product\",\n          type: \"integer\"\n        },\n        productName: %{\n          description: \"Name of the product\",\n          type: \"string\"\n        }\n      }\n    }","ref":"Phantom.Tool.JSONSchema.html"},{"type":"function","title":"Phantom.Tool.JSONSchema.build/1","doc":"","ref":"Phantom.Tool.JSONSchema.html#build/1"},{"type":"type","title":"Phantom.Tool.JSONSchema.json/0","doc":"","ref":"Phantom.Tool.JSONSchema.html#t:json/0"},{"type":"type","title":"Phantom.Tool.JSONSchema.t/0","doc":"","ref":"Phantom.Tool.JSONSchema.html#t:t/0"},{"type":"function","title":"Phantom.Tool.JSONSchema.to_json/1","doc":"","ref":"Phantom.Tool.JSONSchema.html#to_json/1"},{"type":"module","title":"Phantom.Tracker","doc":"Track open streams so that notifications and requests can be sent to clients.\n\nFor example, a request may need to elicit more input from the client, so the\nfirst request stream will remain open, and the notification stream will send\nand new request to the client, and the client will POST its response. The\nnew response connection will notify the first request connection with the result\nand the tool can continue with the elicited information.\n\nSee `m:Phantom#module-persistent-streams` section for more information.","ref":"Phantom.Tracker.html"},{"type":"function","title":"Phantom.Tracker.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Phantom.Tracker.html#child_spec/1"},{"type":"function","title":"Phantom.Tracker.get_request/1","doc":"Fetch the PID of the open request by ID","ref":"Phantom.Tracker.html#get_request/1"},{"type":"function","title":"Phantom.Tracker.get_session/1","doc":"Fetch the PID of the open session by ID","ref":"Phantom.Tracker.html#get_session/1"},{"type":"function","title":"Phantom.Tracker.list_requests/0","doc":"Return a list of all open requests","ref":"Phantom.Tracker.html#list_requests/0"},{"type":"function","title":"Phantom.Tracker.list_resource_listeners/0","doc":"Return a list of all listening for resources","ref":"Phantom.Tracker.html#list_resource_listeners/0"},{"type":"function","title":"Phantom.Tracker.list_sessions/0","doc":"Return a list of all open sessions","ref":"Phantom.Tracker.html#list_sessions/0"},{"type":"function","title":"Phantom.Tracker.notify_prompt_list/0","doc":"Notify any listening MCP sessions that the list of prompts has updated","ref":"Phantom.Tracker.html#notify_prompt_list/0"},{"type":"function","title":"Phantom.Tracker.notify_resource_list/0","doc":"Notify any listening MCP sessions that the list of prompts has updated","ref":"Phantom.Tracker.html#notify_resource_list/0"},{"type":"function","title":"Phantom.Tracker.notify_resource_updated/1","doc":"Notify any listening MCP sessions that the resource has updated","ref":"Phantom.Tracker.html#notify_resource_updated/1"},{"type":"function","title":"Phantom.Tracker.notify_tool_list/0","doc":"Notify any listening MCP sessions that the list of tools has updated","ref":"Phantom.Tracker.html#notify_tool_list/0"},{"type":"function","title":"Phantom.Tracker.requests_topic/0","doc":"","ref":"Phantom.Tracker.html#requests_topic/0"},{"type":"function","title":"Phantom.Tracker.resource_subscription_topic/0","doc":"","ref":"Phantom.Tracker.html#resource_subscription_topic/0"},{"type":"function","title":"Phantom.Tracker.sessions_topic/0","doc":"","ref":"Phantom.Tracker.html#sessions_topic/0"},{"type":"function","title":"Phantom.Tracker.subscribe_resource/1","doc":"Subscribe the process to resource notifications from the PubSub on topic \"phantom:resources\"","ref":"Phantom.Tracker.html#subscribe_resource/1"},{"type":"function","title":"Phantom.Tracker.track_request/3","doc":"Track a request PID","ref":"Phantom.Tracker.html#track_request/3"},{"type":"function","title":"Phantom.Tracker.track_session/3","doc":"Track a session PID","ref":"Phantom.Tracker.html#track_session/3"},{"type":"function","title":"Phantom.Tracker.unsubscribe_resource/1","doc":"Unsubscribe the process to resource notifications from the PubSub on topic \"phantom:resources\"","ref":"Phantom.Tracker.html#unsubscribe_resource/1"},{"type":"function","title":"Phantom.Tracker.untrack/1","doc":"Untrack the processe for everything","ref":"Phantom.Tracker.html#untrack/1"},{"type":"function","title":"Phantom.Tracker.untrack_request/1","doc":"Untrack any processes for the request","ref":"Phantom.Tracker.html#untrack_request/1"},{"type":"function","title":"Phantom.Tracker.untrack_session/1","doc":"Untrack any processes for the session","ref":"Phantom.Tracker.html#untrack_session/1"},{"type":"exception","title":"Phantom.ErrorWrapper","doc":"Wraps errors that occur during a request or batch or requests.\nThis allows the connection to finish, and then reraises with this error\ncontaining the exceptions by request.","ref":"Phantom.ErrorWrapper.html"},{"type":"function","title":"Phantom.ErrorWrapper.new/2","doc":"","ref":"Phantom.ErrorWrapper.html#new/2"},{"type":"extras","title":"CHANGELOG","doc":"","ref":"changelog.html"},{"type":"extras","title":"Unreleased - CHANGELOG","doc":"- Move logging functions from `Phantom.Session` into `Phantom.ClientLogger`.\n- Rename `Phantom.Tracker` functions to be clearer and more straightforward.\n- Consolidate distributed logic into `Phantom.Tracker` such as PubSub topics.\n- Add ability to add tools, prompts, resources in runtime easily. You can call\n  `Phantom.Cache.add_tool(router_module, tool_spec)`. The spec can be built with\n  `Phantom.Tool.build/1`, the function takes a very similar shape to the corresponding macro from `Phantom.MCP.Router`. This will also trigger notifications to clients of tool or prompt list updates.\n- Handle paginatin for 100+ tools and prompts.\n- Change `connect/2` callback to receive request headers and query params from the Plug adapter. The signature is now `%{headers: list({header, value}), params: map()}` where before it was just `list({header, value})`.\n- `Phantom.Tool.build`, `Phantom.Prompt.build` and `Phantom.ResourceTemplate.build` now do more and the `Phantom.Router` macros do less. This is so runtime can have a consistent experience with compiled declarations. For example, you may `Phantom.ResourceTemplate.build(...)` with the same arguments as you would with the router macros, and then call `Phantom.Cache.add_resource_template(...)` and have the same affect as using the `resource ...` macro in a `Phantom.Router` router.\n- Fixed building tool annontations.\n- Fixed resource subscription response and implemented unsubscribe method.\n- Improve documentation","ref":"changelog.html#unreleased"},{"type":"extras","title":"0.2.3 - CHANGELOG","doc":"- Fix the `initialize` request status code and headers. In 0.2.2 it worked\nwith mcp-inspector but not with Claude Desktop or Zed. Now it works with all.","ref":"changelog.html#0-2-3"},{"type":"extras","title":"0.2.2 - CHANGELOG","doc":"- Fix the `initialize` request. It should have kept the SSE stream open.\n- Fix bugs","ref":"changelog.html#0-2-2"},{"type":"extras","title":"0.2.1 - CHANGELOG","doc":"- Fix default `list_resources/2` callback and default implementation.","ref":"changelog.html#0-2-1"},{"type":"extras","title":"0.2.0 - CHANGELOG","doc":"Phantom MCP released!","ref":"changelog.html#0-2-0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}